import unittest
from rma.jemalloc import Jemalloc


class JemallocTestCase(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_align(self):
        data = [[79, 80], [253, 256], [512, 512], [513, 768], [3645, 3840], [4098, 8192], [4200304, 8388608]]
        for given, expected in data:
            self.assertEqual(Jemalloc.align(given), expected)



import unittest
from tests.jemalloc import JemallocTestCase
from tests.simple_splitter import SplitterTestCase
from tests.helpers.helpers import HelpersTestCase


def all_tests():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(JemallocTestCase))
    suite.addTest(unittest.makeSuite(SplitterTestCase))
    suite.addTest(unittest.makeSuite(HelpersTestCase))
    return suite



import logging
from rma.redis import dict_overhead
from redis.exceptions import ResponseError


class GlobalKeySpace(object):
    def __init__(self, redis):
        """
        :param RmaRedis redis:
        :return:
        """
        self.redis = redis
        self.logger = logging.getLogger(__name__)

    def analyze(self):
        total_keys = self.redis.dbsize()
        used = {}
        info = {}

        try:
            for key, value in self.redis.config_get("*max-*-*").items():
                used[key] = value
        except ResponseError as e:
            self.logger.warning("*max* option skipped: %s", repr(e))

        for key, value in self.redis.info('memory').items():
            info[key] = value

        return {
            "info": info,
            "used": used,
            "totalKeys": total_keys,
            "redisKeySpaceOverhead": dict_overhead(total_keys),
        }



from tabulate import tabulate


class TextReporter:
    def print(self, data):

        for report in data:
            if "nodes" in report:
                self.print_nodes(report['nodes'])
            elif "keys" in report:
                self.print_keys(report['keys'])
            elif "stat" in report:
                self.print_keys_stat(report['stat'])
            else:
                self.print_unsupported(report)


    def print_nodes(self, nodes):
        values = []
        print("\r\nNodes\r\n")

        for node in nodes:
            for key, value in node.items():
                if isinstance(value, dict):
                    for sub_key, sub_value in value.items():
                        values.append(["{0} {1}".format(key, sub_key), sub_value])

            for key, value in node.items():
                if not isinstance(value, dict):
                    values.append([key, value])

            print(tabulate(values, ["Stat", "Value"], floatfmt=".2f", tablefmt="pipe"))


    def print_keys(self, keys):
        print("\r\nKeys by types\r\n")
        print(tabulate(keys['data'], keys['headers'], floatfmt=".2f", tablefmt="pipe"))


    def print_keys_stat(self, keys):
        print("\r\nKeys statistic")

        for type, prop in keys.items():
            print("\r\nStat by <{0}>\r\n".format(type))
            print(tabulate(prop['data'], prop['headers'], floatfmt=".2f", tablefmt="pipe"))


    def print_unsupported(self, data):
        print("\r\nUnsupported block")
        print(data)



import json
import sys


class JsonReporter:
    def print(self, data):
        output = {}

        for index, report in enumerate(data):
            if "nodes" in report:
                output['nodes'] = report['nodes']
            elif "keys" in report:
                output['keys'] = self.prepare_keys(report['keys'])
            elif "stat" in report:
                output['stat'] = self.prepare_stat(report['stat'])
            else:
                output["unsupported{0}".format(index)] = report

        print(json.dumps(output, indent=4, sort_keys=True))


    def prepare_keys(self, keys):
        data = []

        for values in keys['data']:
            entry = {}
            for index, value in enumerate(values):
                entry[keys['headers'][index]] = value

            data.append(entry)

        return data


    def prepare_stat(self, params):
        stat = {}

        for type, prop in params.items():
            data = []
            for row in prop['data']:
                entry = {}
                for index, value in enumerate(row):
                    entry[prop['headers'][index]] = value

                data.append(entry)
            stat[type] = data

        return stat



import unittest
from rma.helpers import *


class HelpersTestCase(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_total_row(self):
        data = [
            [1, 2, 'abc'], [4, 3, 'abc']
        ]
        self.assertEqual(make_total_row(data, [sum, sum, 'test']), [5, 5, 'test'])

    def test_pref_encoding(self):
        self.assertEqual('c [50.0%] / a [33.3%] / b [16.6%]', pref_encoding(['a', 'a', 'b', 'c', 'c', 'c']))
        self.assertEqual('c [60.0%] / aaa [40.0%]', pref_encoding(['aaa', 'aaa', 'c', 'c', 'c']))
